#
# fishing-rust-cpp.R, 29 Jan 23
#
# Data from
# https://quick-lint-js.com/blog/cpp-vs-rust-build-times/
# strager.nds@gmail.com

library("colorspace")
library("plyr")


par(bty="l")
par(las=1)
par(pch="+")

pal_col=rainbow(4)

stra=read.csv("strapurp.csv", as.is=TRUE)
stra$host="Linux"
stra$duration=stra$duration_ns/1e6

plot(count(round(stra$duration/100)/10), log="xy", col=pal_col[1],
	xlab="Time (seconds)", ylab="Runs")

# Breakout options from column names
stra$copies=as.numeric(sub("^.+-", "", stra$project))
stra$copies[is.na(stra$copies)]=1

stra$nc_project=sub("-[0-9]+$", "", stra$project)
stra$language=sub("-.+$", "", stra$nc_project)
stra$layout=sub("-.+$", "", sub("^.{3,4}-", "", stra$nc_project))
stra$tests=sub("^.{,12}-", "", sub("^.{3,4}-", "", stra$nc_project))

stra$Date=as.POSIXct(stra$created, origin = "1970-01-01", tz = "GMT")
stra$start_sec=stra$created-min(stra$created)
stra$is_first=FALSE
stra$is_first[stra$id == 1]=TRUE

table(stra$language)

# Time line of language being benchmarked
plot(stra$start_sec, (stra$language == "rust"), col=pal_col[1+(stra$language == "rust")],
	xaxs="i", yaxt="n",
	xlab="Seconds since start", ylab="")
axis(2, at=c(0, 1), label=c("C++", "Rust"))

mos=read.csv("strammer.csv", as.is=TRUE)
mos$duration=mos$duration_ns/1e6
mos$language=mos$project

count(count(stra$id)$freq)
#    x freq
# 1  2    1
# 2  3   28
# 3 10  134

stra_mod=glm(duration ~ copies+language+layout+tests+benchmark+toolchain+start_sec, data=stra)
summary(stra_mod)

# strm_mod=glm(log(duration) ~ project+benchmark+is_first, data=stra)
strm_mod=glm(log(duration) ~ benchmark, data=stra)
strm_mod=glm(log(duration) ~ copies*language+benchmark, data=stra)

strm_mod=glm(log(duration) ~ copies*language+layout+tests+benchmark, data=stra)
summary(strm_mod)

exp(coef(strm_mod))

size=c(1,8,16,32)
rc=function(L) size*(0.028+0.035*L)-0.084*L
rc(1)
# [1] -0.021  0.420  0.924  1.932
exp(rc(1))
# [1] 0.979219 1.521962 2.519348 6.903303
exp(rc(0))
# [1] 1.028396 1.251071 1.565179 2.449784

# scaling of input * language
# C++
# exp(0.0283491)*c(1, 8, 16, 32)
# Rust
# exp(0.0283491+0.0156146)*c(1, 8, 16, 32)

# No copy-paste benchmarks
mos_mod=glm(log(duration) ~ benchmark, data=mos)
mos_mod=glm(log(duration) ~ language+benchmark, data=mos)

mos_mod=glm(log(duration) ~ project+toolchain+benchmark, data=mos)
summary(mos_mod)

exp(coef(mos_mod))


ct=exp(rc(0))
rt=exp(rc(1))

plot(size, ct, type="b", col=pal_col[2],
	ylim=c(0.8, 7),
	xlab="Size", ylab="Multiplication factor")
points(size, rt, type="b", col=pal_col[1])

st_mod=glm(rt ~ size+I(size^2))
summary(st_mod)

st_mod=nls(rt ~ a+b*size^c, start=c(a=1, b=0.1, c=2), trace=TRUE)
summary(st_mod)

ct_mod=glm(ct ~ size+I(size^2))
summary(ct_mod)

ct_mod=nls(ct ~ a+b*size^c, start=c(a=1, b=0.1, c=2), trace=TRUE)
summary(ct_mod)

